/*
    This file is part of audtty, copyright 2008-2010 Chris Taylor.

    audtty is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    audtty is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with audtty; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _BROWSE_H_

#define _BROWSE_H_

void file_browser(gint height);

typedef struct {
	WINDOW *location;
	WINDOW *title;
	WINDOW *list;
} win;


typedef struct {
        gchar *file;
        gchar *title;
} files;


typedef struct {
	gint height;
        gint length;
	gint prev_pos;
	gint pos_height;
	gint first;
	gint selector;
	gchar *location;
        files *files;
} controller;


#endif